From f929fee810350b7b6f62f7c56f1f29bce4bc759f Mon Sep 17 00:00:00 2001
From: Jonathan Rose <jrose@digium.com>
Date: Fri, 13 Nov 2015 13:59:31 -0600
Subject: [PATCH] chan_sip: add TCP/TLS keepalive option to TCP/TLS server

Adds TCP keep alive option to TCP and TLS server sockets. Previously
this option was only being set on session sockets.
http://www.tldp.org/HOWTO/html_single/TCP-Keepalive-HOWTO/
According to the page above, the SO_KEEPALIVE option is useful for
knowing when a TCP connected endpoint has severed communication
without indicating it or has become unreachable for some reason.
Hiroaki Komatsu reported that keep alive was disabled on a socket
and this resulted in a thread listening on for TCP becoming stuck in
a waiting state.

ASTERISK-25364
Reported by: Hiroaki Komatsu

Change-Id: Ie51447db7d4b28cb73e960fc5cf8fa57df62cdc7
---

diff --git a/channels/chan_sip.c b/channels/chan_sip.c
index f678075..62f41d3 100644
--- a/channels/chan_sip.c
+++ b/channels/chan_sip.c
@@ -32105,6 +32105,12 @@
 		ast_log(LOG_ERROR, "SIP TCP Server start failed. Not listening on TCP socket.\n");
 	} else {
 		ast_debug(2, "SIP TCP server started\n");
+		if (sip_tcp_desc.accept_fd >= 0) {
+			int flags = 1;
+			if (setsockopt(sip_tcp_desc.accept_fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
+				ast_log(LOG_ERROR, "Error enabling TCP keep-alive on sip socket: %s\n", strerror(errno));
+			}
+		}
 	}
 
 	/* Start TLS server if needed */
@@ -32125,6 +32131,13 @@
 			ast_log(LOG_ERROR, "TLS Server start failed. Not listening on TLS socket.\n");
 			sip_tls_desc.tls_cfg = NULL;
 		}
+		if (sip_tls_desc.accept_fd >= 0) {
+			int flags = 1;
+			if (setsockopt(sip_tls_desc.accept_fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
+				ast_log(LOG_ERROR, "Error enabling TCP keep-alive on sip socket: %s\n", strerror(errno));
+				sip_tls_desc.tls_cfg = NULL;
+			}
+		}
 	} else if (sip_tls_desc.tls_cfg->enabled) {
 		sip_tls_desc.tls_cfg = NULL;
 		ast_log(LOG_WARNING, "SIP TLS server did not load because of errors.\n");
